/*
 *		filterstack.c
 */

#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>

#include  "../floatimg.h"
#include  "crapulator.h"
#include  "filterstack.h"

// #undef  DEBUG_LEVEL
// #define DEBUG_LEVEL 1

/* -------------------------------------------------------------- */

extern int		verbosity;

static	FilterStack 	f_stacks[NUMBER_OF_STACK];

/* -------------------------------------------------------------- */
int filterstack_init(int numid, int notused)
{
#if DEBUG_LEVEL
fprintf(stderr, ">>> %s ( %d %d )\n", __func__, numid, notused);
#endif

if (numid < 0 || numid > NUMBER_OF_STACK) {
	fprintf(stderr, "%s: slot number %d invalid\n", __func__, numid);
	exit(1);
	}

memset(&f_stacks[numid], 0, sizeof(FilterSlot));

return 0;
}
/* -------------------------------------------------------------- */
int filterstack_add(int numid, int code, int ival, float fval)
{
int		idxsl;
#if DEBUG_LEVEL
fprintf(stderr, ">>> %s ( %d %d %f )\n", __func__, code, ival, fval);
#endif

if (numid < 0 || numid > NUMBER_OF_STACK) {
	fprintf(stderr, "%s: slot number %d invalid\n", __func__, numid);
	exit(1);
	}

if (f_stacks[numid].count == FILTER_BY_STACK) {
	fprintf(stderr, "%s: stack is full\n", __func__);
	return -1;
	}

idxsl = f_stacks[numid].count;			/* aliasing */

f_stacks[numid].slots[idxsl].numero = code;
f_stacks[numid].slots[idxsl].ival   = ival;
f_stacks[numid].slots[idxsl].fval   = fval;

f_stacks[numid].count++;

return 0;
}
/* -------------------------------------------------------------- */
int filterstack_list(int numid, const char *txt)
{
int		idx;

if (numid < 0 || numid > NUMBER_OF_STACK) {
	fprintf(stderr, "%s: slot number %d invalid\n", __func__, numid);
	exit(1);
	}

fprintf(stderr, "--- %2d -- %-20s --------\n", numid, txt);
// fprintf(stderr, "stack at %p, size %d, current %d\n",
//			f_slots, nbre_filters, idx_slot); 
fprintf(stderr, "idx     fx#   name          ival   fval\n");

for (idx=0; idx<f_stacks[numid].count; idx++) {

	fprintf(stderr, "%3d    %3d    %-10s  %3d      %f\n", idx,
			f_stacks[numid].slots[idx].numero,
	crap_name_from_number(f_stacks[numid].slots[idx].numero),
			f_stacks[numid].slots[idx].ival,
			f_stacks[numid].slots[idx].fval);

	}

return 0;
}
/* -------------------------------------------------------------- */
int filterstack_run(int numid, FloatImg *target, int notused)
{
int		idx, foo, eff;
float		fv;

#if DEBUG_LEVEL
fprintf(stderr, ">>> %s ( %p %d )\n", __func__, target, notused);
#endif

if (numid < 0 || numid > NUMBER_OF_STACK) {
	fprintf(stderr, "%s: slot number %d invalid\n", __func__, numid);
	exit(1);
	}

for (idx=0; idx<f_stacks[numid].count; idx++) {

	eff = f_stacks[numid].slots[idx].numero;
	fv  = f_stacks[numid].slots[idx].fval;

	if (verbosity > 1)
		fprintf(stderr, "%d : effect %2d on %p\n",
				idx, eff, target);

	foo = crapulator(target, eff, fv);

	if (foo) {
		fprintf(stderr, "crapulator error %d\n", foo);
		return foo;
		}
	}

return 0;
}
/* -------------------------------------------------------------- */
int load_stack_from_file(int numid, char *fname, int notused)
{
FILE		*fp;
// int		a, b;
// float		f;
// char		line[100];

#if DEBUG_LEVEL
fprintf(stderr, ">>> %s ( '%s' %d )\n", __func__, fname, notused);
#endif

if (numid < 0 || numid > NUMBER_OF_STACK) {
	fprintf(stderr, "%s: slot number %d invalid\n", __func__, numid);
	exit(1);
	}

if (NULL==(fp=fopen(fname, "r"))) {
	perror(fname);
	return -1;
	}

/*
 *		here was dragons
 */
		/* hadoc parser ? */

fclose(fp);

return -1;
}
/* -------------------------------------------------------------- */
int parse_filter_chain(int numid, char *argument)
{
char		*cptr;
int		value, foo;

if (numid < 0 || numid > NUMBER_OF_STACK) {
	fprintf(stderr, "%s: slot number %d invalid\n", __func__, numid);
	exit(1);
	}

fprintf(stderr, "\n%s: arg = '%s'\n", __func__, argument);

foo = filterstack_init(numid, 8);
if (foo) {
	fprintf(stderr, "%s: filterstack init --> %d\n", __func__, foo);
	return foo;
	}

for (;;) {
	cptr = strtok(argument, ":");
	// fprintf(stderr, "cptr %p\n", cptr);
	if (NULL==cptr)	break;
	argument = NULL;
	// fprintf(stderr, "  parsing '%s'\n", cptr);
	if (1 == sscanf(cptr, "%d", &value)) {
		foo = filterstack_add(numid, value, 1, 1.0);
		if (foo) {
			fprintf(stderr, "%s: err %d add\n",
					__func__, foo);
			}
		}
	}

if (verbosity) filterstack_list(numid, __func__);
return 0;
}
/* ----------------------------------------------------------- */
